/*
 * This file is part of the rtap localisation project.
 */


#include "../librtaputil/rtaputil.h"
#include <signal.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <errno.h>
#include <sys/time.h>
#include <time.h>
#include <pthread.h>


#define AGGREGATE_TIMEOUT 500 // Timeout d'agrégation (en mili-secondes)
#define KEEP_TIMEOUT 2000 // Temps que l'on conserve les données dans la liste (en mili-secondes)
#define CHECK_INTERVAL 500000 // Temps entre deux vérifications de la liste (en micro-secondes)

/* Codes d'erreur */
#define ERR_NO_MESSAGE_RECEIVED 1 // Erreur lors de la lecture sur la socket
#define ERR_CREATING_SOCKET 2 // Erreur lors de la création de la socket d'écoute
#define ERR_BAD_USAGE 3 // Mauvais appel au programme


/* Liste chaînée des informations concernant un couple MAC / séquence */
typedef struct _couple_info_list
{
  unsigned char ap_mac_addr_bytes[6] ; // Adresse MAC de l'AP émetteur de l'info en octets
  unsigned char antenna_signal_dbm ; // Puissance du signal reçu par l'AP du mobile
  struct _couple_info_list *next ;
} couple_info_list ;


/* Liste chaînée des couples MAC / séquence */
typedef struct _couple_list
{
  unsigned char mobile_mac_addr_bytes[6] ; // Adresse MAC du mobile en octets
  struct timeval request_time ; // Numéro de séquence de la demande de localisation du mobile (heure de la demande sur le mobile)
  struct timeval start_time ; // Heure d'arrivée du premier paquet du couple
  couple_info_list *info ; // Liste des informations pour ce couple
  struct _couple_list *next ;
  /* Données pour la calibration */
  float x_position ;
  float y_position ;
  float z_position ;
  DIRECTION direction ; // Orientation de la demande de localisation
} couple_list ;


/* En-têtes de fonctions */
void got_couple_info(couple_list **couples, couple_message message) ;
void free_couple_list(couple_list **couples) ;
void print_couple_list(couple_list *couples) ;
void print_couple_info(couple_info_list *info) ;
void* monitor_couple_list(couple_list **couples) ;
void print_usage(char *prog) ;
